#- Licensed Materials - Property of IBM Corp.
#- IBM UrbanCode Deploy
#- (c) Copyright IBM Corporation 2014, 2015. All Rights Reserved.
#-
#- U.S. Government Users Restricted Rights - Use, duplication or disclosure restricted by
#- GSA ADP Schedule Contract with IBM Corp.

from org.codehaus.jettison.json import JSONObject, JSONArray
import sys;
from utilities import Util
from Property import Property
from WASConfLog import Log
from KeyStore import KeyStore
from KeyManager import KeyManager
from TrustManager import TrustManager
from SSLConfig import SSLConfig
from SSLConfigGroup import SSLConfigGroup
from KeySet import KeySet
from SSLConfigRef import SSLConfigRef
from KeySetGroup import KeySetGroup
from JAASConfiguration import JAASConfiguration
from AuthMechanism import AuthMechanism
from JAASAuthData import JAASAuthData
from Certificate import Certificate
from CACertificate import CACertificate
from LDAPUserRegistry import LDAPUserRegistry
from LocalOSUserRegistry import LocalOSUserRegistry
from CustomUserRegistry import CustomUserRegistry
from WIMUserRegistry import WIMUserRegistry
from UserRegistryRef import UserRegistryRef
from TrustedAuthenticationRealm import TrustedAuthenticationRealm

__name_att__ = None;

def export(objid, parentcontainmentpath, parentrespath):

  containmentpath = "%(parentconpath)sSecurity:/" % {'parentconpath':parentcontainmentpath}
  if not containmentpath.startswith("/"):
    containmentpath = "/" + containmentpath;

  dict = _export(objid, parentrespath, "Security");
  dict['conpath'] = containmentpath;
  return dict

def _export(objid, parentrespath, name = ""):
  Util.pushPathElement('Security')


  if parentrespath == "/":
    parentrespath = "";
  respath = "%(parentrespath)s/%(name)s" % { 'parentrespath':parentrespath, 'name':name }
  if not respath.startswith("/"):
    respath = "/" + respath;

  exportedObject = JSONObject();
  exportedObject.put("name", name);
  exportedObject.put("path", respath);
  exportedObject.put("teamMappings", JSONArray());
  exportedObject.put("inheritTeam", "true");
  exportedObject.put("description", "Discovered WebSphereSecurity");
  exportedObject.put("roleName", "WebSphereSecurity");

  roleProperties = JSONObject();

  Util.addAttributeToJsonIfNotNone(roleProperties, "websphere.security.allowbasicauth", objid, "allowBasicAuth","Security");
  Util.addAttributeToJsonIfNotNone(roleProperties, "websphere.security.uselocalsecurityserver", objid, "useLocalSecurityServer","Security");
  Util.addAttributeToJsonIfNotNone(roleProperties, "websphere.security.usedomainqualifiedusernames", objid, "useDomainQualifiedUserNames","Security");
  Util.addAttributeToJsonIfNotNone(roleProperties, "websphere.security.enabled", objid, "enabled","Security");
  Util.addAttributeToJsonIfNotNone(roleProperties, "websphere.security.cachetimeout", objid, "cacheTimeout","Security");
  Util.addAttributeToJsonIfNotNone(roleProperties, "websphere.security.enforcefinegrainedjcasecurity", objid, "enforceFineGrainedJCASecurity","Security");
  Util.addAttributeToJsonIfNotNone(roleProperties, "websphere.security.enablejava2secruntimefiltering", objid, "enableJava2SecRuntimeFiltering","Security");
  Util.addAttributeToJsonIfNotNone(roleProperties, "websphere.security.enforcejava2security", objid, "enforceJava2Security","Security");
  Util.addAttributeToJsonIfNotNone(roleProperties, "websphere.security.allowallpermissionforapplication", objid, "allowAllPermissionForApplication","Security");
  Util.addAttributeToJsonIfNotNone(roleProperties, "websphere.security.appenabled", objid, "appEnabled","Security");
  Util.addAttributeToJsonIfNotNone(roleProperties, "websphere.security.dynamicallyupdatesslconfig", objid, "dynamicallyUpdateSSLConfig","Security");
  Util.addAttributeToJsonIfNotNone(roleProperties, "websphere.security.activeprotocol", objid, "activeProtocol","Security");
  Util.addAttributeToJsonIfNotNone(roleProperties, "websphere.security.internalserverid", objid, "internalServerId","Security");
  Util.addAttributeToJsonIfNotNone(roleProperties, "websphere.security.issuepermissionwarning", objid, "issuePermissionWarning","Security");
  exportedObject.put("roleProperties", roleProperties);
  result = { 'object': exportedObject, 'respath':respath }
  extraObjects = [];
  typeFolders = {};
  extraObjects = Property.exportProperties(objid, respath, extraObjects, typeFolders, 'properties',"Security","PropertyAttrs");
  extraObjects = Property.exportProperties(objid, respath, extraObjects, typeFolders, 'additionalSecAttrs',"Security","AdditionalSecAttrs");
  extraObjects = Property.exportProperties(objid, respath, extraObjects, typeFolders, 'webAuthAttrs',"Security","WebAuthAttrs");
  extraObjects = AuthMechanism.exportAuthMechanisms(objid, respath, extraObjects, typeFolders, 'authMechanisms', "Security", "AuthMechanism");

  applicationLoginConfig = Util.getOptionalAttribute(objid, 'applicationLoginConfig', 'Security');
  if applicationLoginConfig is not None and len(applicationLoginConfig) > 0:
    returndict = Util.createTypeFolder(respath, "ApplicationLoginConfig", typeFolders);
    currespath = returndict['path'];
    if returndict.has_key('object'):
      Util.addFromExport(extraObjects, returndict['object']);
    Util.addAllFromExport(extraObjects, JAASConfiguration._export(applicationLoginConfig, currespath, "JAASConfiguration"));

  systemLoginConfig = Util.getOptionalAttribute(objid, 'systemLoginConfig', 'Security');
  if systemLoginConfig is not None and len(systemLoginConfig) > 0:
    returndict = Util.createTypeFolder(respath, "SystemLoginConfig", typeFolders);
    currespath = returndict['path'];
    if returndict.has_key('object'):
      Util.addFromExport(extraObjects, returndict['object']);
    Util.addAllFromExport(extraObjects, JAASConfiguration._export(systemLoginConfig, currespath, "JAASConfiguration"));

  i = 0;
  keyStores = Util.parseConfigIdListAttribute(objid, 'keyStores', "Security");
  for keyStore in keyStores:
    if len(keyStore) > 0:
      returndict = Util.createTypeFolder(respath, "KeyStore", typeFolders);
      currespath = returndict['path'];
      if returndict.has_key('object'):
        Util.addFromExport(extraObjects, returndict['object']);
      Util.addAllFromExport(extraObjects, KeyStore._export(keyStore, currespath, "KeyStore%s" % i));
      i = i + 1;

  i = 0;
  keyManagers = Util.parseConfigIdListAttribute(objid, 'keyManagers', "Security");
  for keyManager in keyManagers:
    if len(keyManager) > 0:
      returndict = Util.createTypeFolder(respath, "KeyManager", typeFolders);
      currespath = returndict['path'];
      if returndict.has_key('object'):
        Util.addFromExport(extraObjects, returndict['object']);
      Util.addAllFromExport(extraObjects, KeyManager._export(keyManager, currespath, "KeyManager%s" % i));
      i = i + 1;

  i = 0;
  trustManagers = Util.parseConfigIdListAttribute(objid, 'trustManagers', "Security");
  for trustManager in trustManagers:
    if len(trustManager) > 0:
      returndict = Util.createTypeFolder(respath, "TrustManager", typeFolders);
      currespath = returndict['path'];
      if returndict.has_key('object'):
        Util.addFromExport(extraObjects, returndict['object']);
      Util.addAllFromExport(extraObjects, TrustManager._export(trustManager, currespath, "TrustManager%s" % i));
      i = i + 1;

  i = 0;
  repertoires = Util.parseConfigIdListAttribute(objid, 'repertoire', "Security");
  for repertoire in repertoires:
    if len(repertoire) > 0:
      returndict = Util.createTypeFolder(respath, "SSLConfig", typeFolders);
      currespath = returndict['path'];
      if returndict.has_key('object'):
        Util.addFromExport(extraObjects, returndict['object']);
      Util.addAllFromExport(extraObjects, SSLConfig._export(repertoire, currespath, "SSLConfig%s" % i));
      i = i + 1;

  i = 0;
  sslConfigGroups = Util.parseConfigIdListAttribute(objid, 'sslConfigGroups', "Security");
  for sslConfigGroup in sslConfigGroups:
    if len(sslConfigGroup) > 0:
      returndict = Util.createTypeFolder(respath, "SSLConfigGroup", typeFolders);
      currespath = returndict['path'];
      if returndict.has_key('object'):
        Util.addFromExport(extraObjects, returndict['object']);
      Util.addAllFromExport(extraObjects, SSLConfigGroup._export(sslConfigGroup, currespath, "SSLConfigGroup%s" % i));
      i = i + 1;

  i = 0;
  keySets = Util.parseConfigIdListAttribute(objid, 'keySets', "Security");
  for keySet in keySets:
    if len(keySet) > 0:
      returndict = Util.createTypeFolder(respath, "KeySet", typeFolders);
      currespath = returndict['path'];
      if returndict.has_key('object'):
        Util.addFromExport(extraObjects, returndict['object']);
      Util.addAllFromExport(extraObjects, KeySet._export(keySet, currespath, "KeySet%s" % i));
      i = i + 1;

  defaultSSLSettings = Util.getOptionalAttribute(objid, 'defaultSSLSettings', 'Security');
  if defaultSSLSettings is not None and len(defaultSSLSettings) > 0:
    returndict = Util.createTypeFolder(respath, "SSLConfigRef", typeFolders);
    currespath = returndict['path'];
    if returndict.has_key('object'):
      Util.addFromExport(extraObjects, returndict['object']);
    Util.addAllFromExport(extraObjects, SSLConfigRef._export(defaultSSLSettings, currespath, "DefaultSSLSettings"));

  i = 0;
  keySetGroups = Util.parseConfigIdListAttribute(objid, 'keySetGroups', "Security");
  for keySetGroup in keySetGroups:
    if len(keySetGroup) > 0:
      returndict = Util.createTypeFolder(respath, "KeySetGroup", typeFolders);
      currespath = returndict['path'];
      if returndict.has_key('object'):
        Util.addFromExport(extraObjects, returndict['object']);
      Util.addAllFromExport(extraObjects, KeySetGroup._export(keySetGroup, currespath, "KeySetGroup%s" % i));
      i = i + 1;
      
  i = 0;
  cai = 0;
  certificates = Util.parseConfigIdListAttribute(objid, 'certificates',"Security");
  for certificate in certificates:
    if len(certificate) > 0:
      returndict = Util.createTypeFolder(respath, "Certificate", typeFolders);
      currespath = returndict['path'];
      if returndict.has_key('object'):
        Util.addFromExport(extraObjects, returndict['object']);
      if certificate.find("#Certificate_") != -1:
        Util.addAllFromExport(extraObjects, Certificate._export(certificate, currespath, "Certificate%s" % i));
        i = i + 1;
      elif certificate.find("#CACertificate_") != -1:
        Util.addAllFromExport(extraObjects, CACertificate._export(certificate, currespath, "CACertificate%s" % i));     
        cai = cai + 1;      

  i = 0;
  jaasAuthDatas = Util.parseConfigIdListAttribute(objid, 'authDataEntries', "Security");
  for jaasAuthData in jaasAuthDatas:
    if len(jaasAuthData) > 0:
      returndict = Util.createTypeFolder(respath, "JAASAuthData", typeFolders);
      currespath = returndict['path'];
      if returndict.has_key('object'):
        Util.addFromExport(extraObjects, returndict['object']);
      Util.addAllFromExport(extraObjects, JAASAuthData._export(jaasAuthData, currespath, "JAASAuthData%s" % i));
      i = i + 1;

  i = 0;
  ldapUserRegistries = LDAPUserRegistry.getLDAPUserRegistry(objid);
  ldapUserRegistries = ldapUserRegistries.splitlines();
  for ldapUserRegistry in ldapUserRegistries:
    if len(ldapUserRegistry) > 0:
      returndict = Util.createTypeFolder(respath, "LDAPUserRegistry", typeFolders);
      currespath = returndict['path'];
      if returndict.has_key('object'):
        Util.addFromExport(extraObjects, returndict['object']);
      Util.addAllFromExport(extraObjects, LDAPUserRegistry._export(ldapUserRegistry, currespath, "LDAP User Registry%s" % i));
      i = i + 1;

  i = 0;
  localOSUserRegistries = LocalOSUserRegistry.getLocalOSUserRegistry(objid);
  localOSUserRegistries = localOSUserRegistries.splitlines();
  for localOSUserRegistry in localOSUserRegistries:
    if len(localOSUserRegistry) > 0:
      returndict = Util.createTypeFolder(respath, "LocalOSUserRegistry", typeFolders);
      currespath = returndict['path'];
      if returndict.has_key('object'):
        Util.addFromExport(extraObjects, returndict['object']);
      Util.addAllFromExport(extraObjects, LocalOSUserRegistry._export(localOSUserRegistry, currespath, "Local OS User Registry%s" % i));
      i = i + 1;

  i = 0;
  customUserRegistries = CustomUserRegistry.getCustomUserRegistry(objid);
  customUserRegistries = customUserRegistries.splitlines();
  for customUserRegistry in customUserRegistries:
    if customUserRegistry is not None and len(customUserRegistry) > 0:
      returndict = Util.createTypeFolder(respath, "CustomUserRegistry", typeFolders);
      currespath = returndict['path'];
      if returndict.has_key('object'):
        Util.addFromExport(extraObjects, returndict['object']);
      Util.addAllFromExport(extraObjects, CustomUserRegistry._export(customUserRegistry, currespath, "Custom User Registry%s" % i));
      i = i + 1;

  i = 0;
  wimUserRegistries = WIMUserRegistry.getWIMUserRegistry(objid);
  wimUserRegistries = wimUserRegistries.splitlines();
  for wimUserRegistry in wimUserRegistries:
    if wimUserRegistry is not None and len(wimUserRegistry) > 0:
      returndict = Util.createTypeFolder(respath, "WIMUserRegistry", typeFolders);
      currespath = returndict['path'];
      if returndict.has_key('object'):
        Util.addFromExport(extraObjects, returndict['object']);
      Util.addAllFromExport(extraObjects, WIMUserRegistry._export(wimUserRegistry, currespath, "WIM User Registry%s" % i));
      i = i + 1;

  activeUserRegistry = Util.getOptionalAttribute(objid, 'activeUserRegistry', 'Security');
  if activeUserRegistry is not None and len(activeUserRegistry) > 0:
    returndict = Util.createTypeFolder(respath, "UserRegistryRef", typeFolders);
    currespath = returndict['path'];
    if returndict.has_key('object'):
      Util.addFromExport(extraObjects, returndict['object']);
    type = "";
    if activeUserRegistry.find("#LDAPUserRegistry_") != -1:
      type = "LDAPUserRegistry";
    elif activeUserRegistry.find("#LocalOSUserRegistry_") != -1:
      type = "LocalOSUserRegistry";
    elif activeUserRegistry.find("#CustomUserRegistry_") != -1:
      type = "CustomUserRegistry";
    elif activeUserRegistry.find("#WIMUserRegistry_") != -1:
      type = "WIMUserRegistry";
    realm = "";
    realm = Util.getOptionalAttribute(activeUserRegistry, 'realm', "UserRegistry");
    Util.addAllFromExport(extraObjects, UserRegistryRef._export(activeUserRegistry, currespath, type, realm, type + " - " + realm));

  inbound = Util.getOptionalAttribute(objid, 'inboundTrustedAuthenticationRealm',"Security");
  if inbound is not None and len(inbound) > 0:
    returndict = Util.createTypeFolder(respath, "TrustedAuthenticationRealm", typeFolders);
    currespath = returndict['path'];
    if returndict.has_key('object'):
      Util.addFromExport(extraObjects, returndict['object']);
    Util.addAllFromExport(extraObjects, TrustedAuthenticationRealm._export(inbound, currespath, "inboundTrustedAuthenticationRealm"));  

  outbound = Util.getOptionalAttribute(objid, 'outboundTrustedAuthenticationRealm',"Security");
  if outbound is not None and len(outbound) > 0:
    returndict = Util.createTypeFolder(respath, "TrustedAuthenticationRealm", typeFolders);
    currespath = returndict['path'];
    if returndict.has_key('object'):
      Util.addFromExport(extraObjects, returndict['object']);
    Util.addAllFromExport(extraObjects, TrustedAuthenticationRealm._export(outbound, currespath, "outboundTrustedAuthenticationRealm"));  
    
  if len(extraObjects) != 0:
    result['extraObjects'] = extraObjects;
  Util.popPathElement('Security');
  return result;


def import(containmentpath, roleName, jsonobject):
  if not jsonobject.has("roleProperties"):
    raise Exception("Security resource has no role properties!");

  roleProperties = jsonobject.getJSONObject("roleProperties");

  objid = Util.getid(containmentpath);
  if objid == None or len(objid) == 0:
    index = containmentpath.rindex('/',0,len(containmentpath)-2)+1;
    parentconpath = containmentpath[0:index];
    parentid = Util.getid(parentconpath);
    if parentid == None or len(parentid) == 0:
      raise Exception("Security does not exist to create CoreGroupBridgeSettings on.");
    objid = create(parentid, jsonobject);
  else:
    update(objid,jsonobject);
  return objid;


def create(parentid, jsonobject):
  Util.pushPathElement('Security')
  if not jsonobject.has("roleProperties"):
    raise Exception("Security resource has no role properties!");

  roleProperties = jsonobject.getJSONObject("roleProperties");

  properties = [];
  Util.addIfNotNone(properties, "allowBasicAuth", roleProperties.optString("websphere.security.allowbasicauth", None));
  Util.addIfNotNone(properties, "useLocalSecurityServer", roleProperties.optString("websphere.security.uselocalsecurityserver", None));
  Util.addIfNotNone(properties, "useDomainQualifiedUserNames", roleProperties.optString("websphere.security.usedomainqualifiedusernames", None));
  Util.addIfNotNone(properties, "enabled", roleProperties.optString("websphere.security.enabled", None));
  Util.addIfNotNone(properties, "cacheTimeout", roleProperties.optString("websphere.security.cachetimeout", None));
  Util.addIfNotNone(properties, "enforceFineGrainedJCASecurity", roleProperties.optString("websphere.security.enforcefinegrainedjcasecurity", None));
  Util.addIfNotNone(properties, "enableJava2SecRuntimeFiltering", roleProperties.optString("websphere.security.enablejava2secruntimefiltering", None));
  Util.addIfNotNone(properties, "enforceJava2Security", roleProperties.optString("websphere.security.enforcejava2security", None));
  Util.addIfNotNone(properties, "allowAllPermissionForApplication", roleProperties.optString("websphere.security.allowallpermissionforapplication", None));
  Util.addIfNotNone(properties, "appEnabled", roleProperties.optString("websphere.security.appenabled", None));
  Util.addIfNotNone(properties, "dynamicallyUpdateSSLConfig", roleProperties.optString("websphere.security.dynamicallyupdatesslconfig", None));
  Util.addIfNotNone(properties, "activeProtocol", roleProperties.optString("websphere.security.activeprotocol", None));
  Util.addIfNotNone(properties, "internalServerId", roleProperties.optString("websphere.security.internalserverid", None));
  Util.addIfNotNone(properties, "issuePermissionWarning", roleProperties.optString("websphere.security.issuepermissionwarning", None));
  print "Creating Security with attributes";
  for prop in properties:
    print "%(name)s = %(value)s" % { 'name': prop[0], 'value': prop[1] };
  objid = Util.create("Security", parentid, properties);
  
  jaasAuthDatas = Util.parseConfigIdListAttribute(objid, 'authDataEntries', "Security");
  for jaasAuthData in jaasAuthDatas:
    if len(jaasAuthData) > 0:
      Util.remove(jaasAuthData);

  if jsonobject.has('children'):
    for curjsonobject in Util.getHighestChildrenWithRole(jsonobject.getJSONArray('children')):
      currole = curjsonobject.getString('roleName');
      attName = None;
      currpath = curjsonobject.getString('path');
      lastSlashIndex = currpath.rindex('/');
      nextToLastSlashIndex = currpath.rindex('/', 0, lastSlashIndex - 1);
      foldername = currpath[nextToLastSlashIndex+1:lastSlashIndex];
      if (foldername.find(Util.getTypeFoldername('PropertyAttrs')) > -1):
        attName = 'properties';
      elif (foldername.find(Util.getTypeFoldername('AdditionalSecAttrs')) > -1):
        attName = 'additionalSecAttrs';
      elif (foldername.find(Util.getTypeFoldername('WebAuthAttrs')) > -1):
        attName = 'webAuthAttrs';
      elif (foldername.find(Util.getTypeFoldername('ApplicationLoginConfig')) > -1):
        attName = 'applicationLoginConfig';
      elif (foldername.find(Util.getTypeFoldername('SystemLoginConfig')) > -1):
        attName = 'systemLoginConfig';

      if currole == "WebSphereJAASConfiguration":
        JAASConfiguration.create(objid, curjsonobject, attName);

      propertyObject = 0;
      propertyObject = Property.createObjIfRole(objid, curjsonobject, currole, attName);
      if (propertyObject == 1):
        continue;

      authObject = AuthMechanism.createObjIfRole(objid, curjsonobject, currole);
      if (authObject == 1):
        continue;

      if currole == "WebSphereKeyStore":
        KeyStore.create(objid, curjsonobject);
      elif currole == "WebSphereKeyManager":
        KeyManager.create(objid, curjsonobject);
      elif currole == "WebSphereTrustManager":
        TrustManager.create(objid, curjsonobject);
      elif currole == "WebSphereSSLConfig":
        SSLConfig.create(objid, curjsonobject, "repertoire");
      elif currole == "WebSphereSSLConfigGroup":
        SSLConfigGroup.create(objid, curjsonobject);
      elif currole == "WebSphereKeySet":
        KeySet.create(objid, curjsonobject);
      elif currole == "WebSphereKeySetGroup":
        KeySetGroup.create(objid, curjsonobject);
      elif currole == "WebSphereSSLConfigRef":
        SSLConfigRef.create(objid, curjsonobject, "defaultSSLSettings");
      elif currole == "WebSphereJAASAuthData":
        JAASAuthData.create(objid, curjsonobject);        
      elif currole == "WebSphereCertificate":
        Certificate.create(objid, curjsonobject);   
      elif currole == "WebSphereCACertificate":
        CACertificate.create(objid, curjsonobject);
      elif currole == "WebSphereLDAPUserRegistry":
        LDAPUserRegistry.create(objid, curjsonobject);
      elif currole == "WebSphereLocalOSUserRegistry":
        LocalOSUserRegistry.create(objid, curjsonobject);
      elif currole == "WebSphereCustomUserRegistry":
        CustomUserRegistry.create(objid, curjsonobject);
      elif (currole == "WebSphereTrustedAuthenticationRealm") and (curjsonobject.getString("name") == "inboundTrustedAuthenticationRealm"):
        TrustedAuthenticationRealm.create(objid, curjsonobject, 'inboundTrustedAuthenticationRealm');          
      elif (currole == "WebSphereTrustedAuthenticationRealm") and (curjsonobject.getString("name") == "outboundTrustedAuthenticationRealm"):
        TrustedAuthenticationRealm.create(objid, curjsonobject, 'outboundTrustedAuthenticationRealm');               

  # Must create UserRegistryRef after creating User Registries
  if jsonobject.has('children'):
    for curjsonobject in Util.getHighestChildrenWithRole(jsonobject.getJSONArray('children')):
      currole = curjsonobject.getString('roleName');
      if currole == "WebSphereUserRegistryRef":
        UserRegistryRef.import(objid, curjsonobject);
  Util.popPathElement('Security');


def update(objid,jsonobject):
  Util.pushPathElement('Security')
  if not jsonobject.has("roleProperties"):
    raise Exception("Security resource has no role properties!");

  roleProperties = jsonobject.getJSONObject("roleProperties");

  atts = [];
  Util.addAttIfChanged(objid, atts, roleProperties.optString("websphere.security.allowbasicauth", None), "allowBasicAuth","Security");
  Util.addAttIfChanged(objid, atts, roleProperties.optString("websphere.security.uselocalsecurityserver", None), "useLocalSecurityServer","Security");
  Util.addAttIfChanged(objid, atts, roleProperties.optString("websphere.security.usedomainqualifiedusernames", None), "useDomainQualifiedUserNames","Security");
  Util.addAttIfChanged(objid, atts, roleProperties.optString("websphere.security.enabled", None), "enabled","Security");
  Util.addAttIfChanged(objid, atts, roleProperties.optString("websphere.security.cachetimeout", None), "cacheTimeout","Security");
  Util.addAttIfChanged(objid, atts, roleProperties.optString("websphere.security.enforcefinegrainedjcasecurity", None), "enforceFineGrainedJCASecurity","Security");
  Util.addAttIfChanged(objid, atts, roleProperties.optString("websphere.security.enablejava2secruntimefiltering", None), "enableJava2SecRuntimeFiltering","Security");
  Util.addAttIfChanged(objid, atts, roleProperties.optString("websphere.security.enforcejava2security", None), "enforceJava2Security","Security");
  Util.addAttIfChanged(objid, atts, roleProperties.optString("websphere.security.allowallpermissionforapplication", None), "allowAllPermissionForApplication","Security");
  Util.addAttIfChanged(objid, atts, roleProperties.optString("websphere.security.appenabled", None), "appEnabled","Security");
  Util.addAttIfChanged(objid, atts, roleProperties.optString("websphere.security.dynamicallyupdatesslconfig", None), "dynamicallyUpdateSSLConfig","Security");
  Util.addAttIfChanged(objid, atts, roleProperties.optString("websphere.security.activeprotocol", None), "activeProtocol","Security");
  Util.addAttIfChanged(objid, atts, roleProperties.optString("websphere.security.internalserverid", None), "internalServerId","Security");
  Util.addAttIfChanged(objid, atts, roleProperties.optString("websphere.security.issuepermissionwarning", None), "issuePermissionWarning","Security");
  if len(atts) != 0:
    print "Modifying Security with attributes:"
    for prop in atts:
      print "%(name)s = %(value)s" % { 'name': prop[0], 'value': prop[1] };

    Util.modify(objid, atts);
  else:   
    print "Security configuration up to date.";

  Property.removeProperties(objid, 'properties',"Security");
  Property.removeProperties(objid, 'additionalSecAttrs',"Security");
  Property.removeProperties(objid, 'webAuthAttrs',"Security");
  # AuthMechanism.removeAuthMechanisms(objid, 'authMechanisms',"Security");

  applicationLoginConfig = Util.getOptionalAttribute(objid, 'applicationLoginConfig',"Security");
  if applicationLoginConfig is not None and len(applicationLoginConfig) > 0:
    Util.remove(applicationLoginConfig);

  systemLoginConfig = Util.getOptionalAttribute(objid, 'systemLoginConfig',"Security");
  if systemLoginConfig is not None and len(systemLoginConfig) > 0:
    Util.remove(systemLoginConfig);

  # Cannot delete KeyStores.  We can only update them because we are corrupting the password
  # field when we try to create them.
  # keyStores = Util.parseConfigIdListAttribute(objid, 'keyStores', "Security");
  # for keyStore in keyStores:
  #   Util.remove(keyStore);

  keyManagers = Util.parseConfigIdListAttribute(objid, 'keyManagers', "Security");
  for keyManager in keyManagers:
    Util.remove(keyManager);

  trustManagers = Util.parseConfigIdListAttribute(objid, 'trustManagers', "Security");
  for trustManager in trustManagers:
    Util.remove(trustManager);

  repertoires = Util.parseConfigIdListAttribute(objid, 'repertoire', "Security");
  for repertoire in repertoires:
    Util.remove(repertoire);

  sslConfigGroups = Util.parseConfigIdListAttribute(objid, 'sslConfigGroups', "Security");
  for sslConfigGroup in sslConfigGroups:
    Util.remove(sslConfigGroup);

  # Cannot delete KeySets.  We can only update them because we are corrupting the password
  # field when we try to create them.
  #keySets = Util.parseConfigIdListAttribute(objid, 'keySets', "Security");
  #for keySet in keySets:
  #  Util.remove(keySet);

  keySetGroups = Util.parseConfigIdListAttribute(objid, 'keySetGroups', "Security");
  for keySetGroup in keySetGroups:
    Util.remove(keySetGroup);

  Util.unsetAttribute(objid, "defaultSSLSettings", "Security");
  
  jaasAuthDatas = Util.parseConfigIdListAttribute(objid, 'authDataEntries', "Security");
  for jaasAuthData in jaasAuthDatas:
    if len(jaasAuthData) > 0:
      Util.remove(jaasAuthData);  

  certificates = Util.parseConfigIdListAttribute(objid, 'certificates',"Security");
  for certificate in certificates:
    if len(certificate) > 0:
      if certificate.find("#CACertificate_") != -1:
        Util.remove(certificate);

  inbound = Util.getOptionalAttribute(objid, 'inboundTrustedAuthenticationRealm',"Security");
  if inbound is not None and len(inbound) > 0:
    Util.remove(inbound);  

  outbound = Util.getOptionalAttribute(objid, 'outboundTrustedAuthenticationRealm',"Security");
  if outbound is not None and len(outbound) > 0:
    Util.remove(outbound);  

  # Must create Certificates, TrustManagers, KeyStores and KeyManagers first so that references can be created later.
  if jsonobject.has('children'):
    for curjsonobject in Util.getHighestChildrenWithRole(jsonobject.getJSONArray('children')):
      currole = curjsonobject.getString('roleName');
      if currole == "WebSphereTrustManager":
        TrustManager.create(objid, curjsonobject);
      elif currole == "WebSphereKeyStore":
        keyStoreToUpdate = KeyStore.find(objid, curjsonobject);
        if keyStoreToUpdate is not None or len(keyStoreToUpdate) > 0:
          KeyStore.update(keyStoreToUpdate, curjsonobject);
      elif currole == "WebSphereKeyManager":
        KeyManager.create(objid, curjsonobject);
      elif currole == "WebSphereCertificate":
        certificateToUpdate = Certificate.find(objid, curjsonobject);
        if certificateToUpdate is not None or len(certificateToUpdate) > 0:
          Certificate.update(certificateToUpdate, curjsonobject);
        

  # Must create SSLConfigs and KeySets next so that references can be created later.
  if jsonobject.has('children'):
    for curjsonobject in Util.getHighestChildrenWithRole(jsonobject.getJSONArray('children')):
      currole = curjsonobject.getString('roleName');
      if currole == "WebSphereSSLConfig":
        SSLConfig.create(objid, curjsonobject, "repertoire");
      elif currole == "WebSphereKeySet":
        keySetToUpdate = KeySet.find(objid, curjsonobject);
        if keySetToUpdate is not None or len(keySetToUpdate) > 0:
          KeySet.update(keySetToUpdate, curjsonobject);

  # Must create KeySetGroups next so that references can be created later.
  if jsonobject.has('children'):
    for curjsonobject in Util.getHighestChildrenWithRole(jsonobject.getJSONArray('children')):
      currole = curjsonobject.getString('roleName');
      if currole == "WebSphereKeySetGroup":
        KeySetGroup.create(objid, curjsonobject);

  # Must create User Registries so that references can be created later
  if jsonobject.has('children'):
    for curjsonobject in Util.getHighestChildrenWithRole(jsonobject.getJSONArray('children')):
      currole = curjsonobject.getString('roleName');
      if currole == "WebSphereLDAPUserRegistry":
        LDAPUserRegistry.update(objid, curjsonobject);
      elif currole == "WebSphereLocalOSUserRegistry":
        LocalOSUserRegistry.update(objid, curjsonobject);
      elif currole == "WebSphereCustomUserRegistry":
        CustomUserRegistry.update(objid, curjsonobject);
      elif currole == "WebSphereWIMUserRegistry":
        WIMUserRegistry.update(objid, curjsonobject);

  if jsonobject.has('children'):
    for curjsonobject in Util.getHighestChildrenWithRole(jsonobject.getJSONArray('children')):
      currole = curjsonobject.getString('roleName');
      attName = None;
      currpath = curjsonobject.getString('path');
      lastSlashIndex = currpath.rindex('/');
      nextToLastSlashIndex = currpath.rindex('/', 0, lastSlashIndex - 1);
      foldername = currpath[nextToLastSlashIndex+1:lastSlashIndex];
      if (foldername.find(Util.getTypeFoldername('PropertyAttrs')) > -1):
        attName = 'properties';
      elif (foldername.find(Util.getTypeFoldername('AdditionalSecAttrs')) > -1):
        attName = 'additionalSecAttrs';
      elif (foldername.find(Util.getTypeFoldername('WebAuthAttrs')) > -1):
        attName = 'webAuthAttrs';
      elif (foldername.find(Util.getTypeFoldername('ApplicationLoginConfig')) > -1):
        attName = 'applicationLoginConfig';
      elif (foldername.find(Util.getTypeFoldername('SystemLoginConfig')) > -1):
        attName = 'systemLoginConfig';

      if currole == "WebSphereJAASConfiguration":
        JAASConfiguration.create(objid, curjsonobject, attName);

      propertyObject = 0;
      propertyObject = Property.createObjIfRole(objid, curjsonobject, currole, attName);
      if (propertyObject == 1):
        continue;

      authObject = AuthMechanism.updateObjIfRole(objid, curjsonobject, currole, "Security");
      if (authObject == 1):
        continue;

      if currole == "WebSphereSSLConfigGroup":
        SSLConfigGroup.create(objid, curjsonobject);
      elif currole == "WebSphereSSLConfigRef":
        SSLConfigRef.create(objid, curjsonobject, "defaultSSLSettings");
      elif currole == "WebSphereJAASAuthData":
        JAASAuthData.create(objid, curjsonobject);      
      elif currole == "WebSphereCACertificate":
        CACertificate.create(objid, curjsonobject);
      elif currole == "WebSphereUserRegistryRef":
        UserRegistryRef.import(objid, curjsonobject);
      elif (currole == "WebSphereTrustedAuthenticationRealm") and (curjsonobject.getString("name") == "inboundTrustedAuthenticationRealm"):
        TrustedAuthenticationRealm.create(objid, curjsonobject, 'inboundTrustedAuthenticationRealm');          
      elif (currole == "WebSphereTrustedAuthenticationRealm") and (curjsonobject.getString("name") == "outboundTrustedAuthenticationRealm"):
        TrustedAuthenticationRealm.create(objid, curjsonobject, 'outboundTrustedAuthenticationRealm');               
        
  Util.popPathElement('Security');
